@@ -1,91 +1,67 @@
 package Aspect::Library::Listenable;
 
-# TODO: update docs to explain always_fire, setting handler params
-# specifically instead of getting default (event), and event cloning
-
 use strict;
 use warnings;
-use Carp                               ();
-use Scalar::Util                       ();
-use Sub::Install                       ();
-use Aspect::Modular                    ();
-use Aspect::Advice::Before             ();
-use Aspect::Library::Listenable::Event ();
+use Carp;
+use Scalar::Util qw(weaken);
+use Aspect;
 
-our $VERSION = '0.97_06';
-our @ISA     = 'Aspect::Modular';
+use base qw(Aspect::Modular Exporter);
 
-sub import {
-	my $into = caller();
-
-	Sub::Install::install_sub( {
-		code => $_,
-		into => $into,
-	} ) foreach qw{
-		add_listener
-		remove_listener
-	};
-
-	return 1;
-}
+our @EXPORT = qw(add_listener remove_listener);
 
 sub get_advice {
 	my ($self, $event_name, $pointcut, %event_params) = @_;
-	Aspect::Advice::Before->new(
-		lexical  => $self->lexical,
-		pointcut => $pointcut,
-		code     => sub {
-			my $context    = $_;
-			my $listenable = $context->self;
-			my %params     = %event_params;
+	before {
+		local $_;
 
-			local $_;
-			return unless has_listeners($listenable, $event_name);
+		my $context    = shift;
+		my $listenable = $context->self;
 
-			my $always_fire = delete $params{__always_fire};
-			my %old_state = get_listenable_state($listenable, \%params);
-			$context->original->( $context->args );
-			my %new_state = get_listenable_state($listenable, \%params);
+		return unless has_listeners($listenable, $event_name);
 
-			return if
-				!$always_fire &&
-				keys %old_state &&
-				is_equal_state(\%old_state, \%new_state);
+		my %old_state = get_listenable_state($listenable, \%event_params);
+		$context->run_original($context->params);
+		my %new_state = get_listenable_state($listenable, \%event_params);
 
-			my @params = $context->args;
-			shift @params; # remove $self
-			my $event = Aspect::Library::Listenable::Event->new(
-				name   => $event_name,
-				source => $listenable,
-				params => \@params,
-				%new_state,
-				map {("old_$_" => $old_state{$_})} keys %old_state,
-			);
+		return if
+			keys %old_state &&
+			is_equal_state(\%old_state, \%new_state);
 
-			fire_event($event);
-		},
-	);
+		my @params = $context->params;
+		shift @params; # remove $self
+		my $event = Aspect::Library::Listenable::Event->new(
+			name   => $event_name,
+			source => $listenable,
+			params => \@params,
+			%new_state,
+			map {("old_$_" => $old_state{$_})} keys %old_state,
+		);
+
+		fire_event($event);
+		
+	} $pointcut;
 }
 
 sub add_listener ($$$) {
 	my ($listenable, $event_name, $listener) = @_;
-	Carp::croak "listenable is not a hash based object: [$listenable]"
+	croak "listenable is not a hash based object: [$listenable]"
 		unless is_hash($listenable);
 	my $key = get_listener_key($event_name);
 	$listenable->{$key} = [] unless exists $listenable->{$key};
 	my $listeners = get_listeners($listenable, $event_name);
 	my $lastIndex = (push @$listeners, $listener) - 1;
-	if ( ref $listener eq 'ARRAY' ) { # type 3 listener
-		Scalar::Util::weaken( $listeners->[$lastIndex]->[1] );
-	} elsif ( ref $listener ne 'CODE' ) { # type 2 listener
-		Scalar::Util::weaken( $listeners->[$lastIndex] );
+	if (ref $listener eq 'ARRAY') { # type 3 listener
+		weaken $listeners->[$lastIndex]->[1];
+	} elsif (ref $listener ne 'CODE') { # type 2 listener
+		weaken $listeners->[$lastIndex];
 	}
 }
 
 sub remove_listener ($$$) {
 	my ($listenable, $event_name, $listener) = @_;
 	my $listeners = get_listeners($listenable, $event_name);
-	Carp::croak "listenable has no listeners for event: [$event_name]"
+	croak "listenable has no listeners for event: [$event_name]"
 		unless $listeners;
 	my $oldSize = @$listeners;
 	foreach my $i (0..@$listeners - 1) {
@@ -95,7 +71,7 @@ sub remove_listener ($$$) {
 			last;
 		}
 	}
-	Carp::croak "listener not found: [$event_name, $listener]"
+	croak ";istener not found: [$event_name, $listener]"
 		if $oldSize == @$listeners;
 }
 
@@ -110,21 +86,14 @@ sub fire_event {
 
 sub notify_listener {
 	my ($listener, $event) = @_;
-	local $_;
 	my $clone = $event->clone;
 	my $method_name;
 	if (ref $listener eq 'CODE') {
 		&$listener($clone);
 	} elsif (ref $listener eq 'ARRAY') {
-		my $o = $listener->[1];
-		return unless defined $o; # could be a dead weak ref
 		$method_name = $listener->[0];
-		my @params = $listener->[2]?
-			map { $event->$_ } @{$listener->[2]}:
-			($clone);
-		$o->$method_name(@params);
+		$listener->[1]->$method_name($clone);
 	} else {
-		return unless defined $listener; # could be a dead weak ref
 		$method_name = 'handle_event_'. $event->name;
 		$listener->$method_name($clone);
 	}
@@ -178,52 +147,70 @@ sub is_hash {
 	return $1 eq 'HASH';
 }
 
+
+# event class -----------------------------------------------------------------
+
+package Aspect::Library::Listenable::Event;
+
+sub new {
+	my ($class, %params) = @_;
+	return bless {%params}, $class;
+}
+
+sub AUTOLOAD {
+	my ($self, $value) = @_;
+	my $key  = our $AUTOLOAD;
+	return if $key =~ /DESTROY$/;
+	$key =~ s/^.*:://;
+	return @_ == 1? $self->{$key}: ($self->{$key} = $value);
+}
+
+sub clone { 
+	my $self = shift;
+	my $class = ref $self;
+	my $clone = $class->new;
+	while (my ($key, $value) = each %$self) { $clone->{$key} = $value }
+	return $clone;
+}
+
+sub as_string {
+	my $self = shift;
+	return join ', ', map { "$_:$self->{$_}" } sort keys %$self;
+}
+
 1;
 
-__END__
-
-=pod
-
 =head1 NAME
 
 Aspect::Library::Listenable - Observer pattern with events
 
 =head1 SYNOPSIS
 
-  # The class that we will make listenable
+  # the class that we will make listenable
   package Point;
   
-  sub new {
-     bless { color => 'blue' }, shift;
-  }
   
-  sub erase {
-      print 'erased!';
-  }
+  sub new   { bless {color => 'blue'}, shift }
+  sub erase { print 'erased!' }
   
-  sub get_color {
-      $_[0]->{color};
-  }
+  sub get_color { shift->{color} }
+  sub set_color { shift->{color} = pop }
   
-  sub set_color {
-      $_[0]->{color} = $_[1];
-  }
   
   package main;
-  
   use Aspect;
   use Aspect::Library::Listenable;
   
-  # Setup the simplest listenable relationship: a signal
   
-  # Define the aspect for the listenable relationship
-  aspect Listenable => ( Erase => call 'Point::erase' );
+  # setup the simplest listenable relationship: a signal
   
-  # Now add a listener
-  my $erase_listener = sub { print shift->as_string };
-  add_listener $point, Erase => $erase_listener;
+  # define the aspect for the listenable relationship
+  aspect Listenable => (Erase => call 'Point::erase');
+  
+  # now add a listener
+  add_listener $point, Erase =>
+     my $erase_listener = sub { print shift->as_string };
   
-  my $point = Point->new;
   $point->erase;
   # prints: "erased! name:Erase, source:Point"
   
@@ -231,7 +218,8 @@ Aspect::Library::Listenable - Observer pattern with events
   $point->erase;
   # prints: "erased!"
   
-  # A more complex relationship: listeners get old and new color values
+  
+  # a more complex relationship: listeners get old and new color values
   # and will only be notified if these values are not equal
   aspect Listenable =>
      (Color => call 'Point::set_color', color => 'get_color');
@@ -241,11 +229,10 @@ Aspect::Library::Listenable - Observer pattern with events
   
   $point->set_color('red');
   # prints: "name:Color, source:Point, color:red, old_color:blue, params:red"
-
-  $point->set_color('red'); # does not print anything, color unchanged
   
   remove_listener $point, Color => $color_listener;
   
+  
   # listeners can be callback, as above, or they can be objects
   
   package ColorListener;
@@ -272,6 +259,10 @@ Aspect::Library::Listenable - Observer pattern with events
   # prints: "heard an erase event!"
   remove_listener $point, Color => $method_listener;
 
+=head1 SUPER
+
+L<Aspect::Modular>
+
 =head1 DESCRIPTION
 
 A reusable aspect for implementing the Listenable design pattern. It lets
@@ -281,7 +272,7 @@ listenable are called, registered listeners will be notified.
 
 Some examples of use are:
 
-=over 4
+=over4
 
 =item *
 
@@ -299,7 +290,7 @@ any event of any model.
 
 The Listenable pattern is a variation of the basic Observer pattern:
 
-=over 4
+=over4
 
 =item 1
 
@@ -335,7 +326,7 @@ This is similar to how methods and classes are defined only once.
 Each listenable relationship between classes is defined by one aspect,
 answering 3 questions:
 
-=over 4
+=over4
 
 =item 1
 
@@ -361,7 +352,7 @@ You create a listenable aspect so:
 The C<EVENT_DATA> part is optional. The three parameters are your answers
 to the questions above:
 
-=over 4
+=over4
 
 =item EVENT_NAME
 
@@ -409,7 +400,7 @@ Because the aspect should be created only I<Once> during a program run,
 for each listenable relationship type, there are several options for
 choosing the place to actually create it:
 
-=over 4
+=over4
 
 =item *
 
@@ -436,7 +427,7 @@ someone, and not just fired into the void.
 
 =head2 ADDING AND REMOVING LISTENERS
 
-The simplest listener is a C<CODE> ref. It can added and removed so:
+The simplest listener is a C<CODE> ref, and is added and removed like this:
 
   use Aspect::Library::Listenable;
   my $code = sub { print "event!" }
@@ -445,11 +436,9 @@ The simplest listener is a C<CODE> ref. It can added and removed so:
   remove_listener $point, Color => $code; # remove
   $point->set_color('yellow');            # event will not fire
 
-The event object is the only parameter received by the callback.
-
 The other two types of listeners are I<object>, and I<method>:
 
-=over 4
+=over4
 
 =item 1
 
@@ -471,45 +460,22 @@ When the listener is an array ref (method listener), the method name
 type of listener, you do not remove the array ref but the listener
 object, i.e. exactly like you remove an object listener.
 
-For method listeners, you can also change the parameter list of the
-method. Usually, the event is the only parameter to the listener method.
-By changing the parameter list, you can turn any existing method into a
-listener method, without changing it.
-
-You change the parameter list, by providing a list of event properties,
-whose values will become the new parameter list. Here is how to make a
-C<Family::set_father_name> run each time C<Person::set_name> is called on
-the father object:
-
-  aspect Listenable => (
-     NameChange => call 'Person::set_name',
-     name => 'name',
-  );
-
-  $father = Person->new;
-  $family = Family->new;
-
-  add_listener $father, NameChange =>
-     [set_father_name => $family, [qw(name)]];
-
-  $father->set_name('dan'); # $family->set_father_name('dan') will be called
-
 =head2 HANDLING EVENTS
 
 Listener code is called with one parameter: the event. Its class is
 C<Aspect::Listenable::Event>. All events have at least these properties:
 
-=over 4
+=over4
 
-=item C<name>
+=item name
 
 The name of the event as defined in the aspect.
 
-=item C<source>
+=item source
 
 The listenable object.
 
-=item C<params>
+=item params
 
 The event was fired because a method was called. In this property you
 will find an array ref of the parameters sent to that method.
@@ -532,7 +498,7 @@ a point after a C<Color> event:
 
 =head1 CAVEATS
 
-=over 4
+=over4
 
 =item *
 
@@ -551,25 +517,16 @@ C<aspect Listenable...> in a void context.
 
 =head1 SEE ALSO
 
+See the L<Aspect|::Aspect> pods for a guide to the Aspect module.
+
+There are several papers on the web about implementing Observer with
+aspects. Here is an example:
+L<http://www.itu.dk/people/jborella/docs/observer.pdf>.
+
+You can see examples of API usage in the unit test for the aspect.
+
 C<Class::Listener>, C<Class::Observable>. Both are object-oriented
 solutions to the same problem. Both force you to change the listenable
 class, by adding the code to fire events inside your "hot" methods.
 
-=head1 AUTHORS
-
-Adam Kennedy E<lt>adamk@cpan.orgE<gt>
-
-Marcel GrE<uuml>nauer E<lt>marcel@cpan.orgE<gt>
-
-Ran Eilam E<lt>eilara@cpan.orgE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2001 by Marcel GrE<uuml>nauer
-
-Some parts copyright 2009 - 2011 Adam Kennedy.
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
-
 =cut
